// Typickm pkladem pouit je loha producenta konzumenta. 
// Nekomunikuj ale plnnm a vyprazdovnm jednoho (kruhovho) bufferu, 
// ale vymn si buffery cel (przdn za pln a opan).

import java.util.*;
import java.util.concurrent.*;   

public class VymenikTest {   

	private static final int FULL = 10;	               // dlka bufferu
	private static final int COUNT = FULL * 12;        // poet dat je 120
	private static final Random random = new Random();
	private static volatile int sum = 0;

	// pedv se seznam celch sel
	private static Exchanger<List<Integer>> exchanger = new Exchanger<List<Integer>>();

	// 2 vymovan buffery
	private static List<Integer> initiallyEmptyBuffer; 
	private static List<Integer> initiallyFullBuffer;	

	// zvora s taem 2
	private static CountDownLatch stopLatch = new CountDownLatch(2);   

	// to je Producent
	private static class FillingLoop implements Runnable {

		public void run() {
			List<Integer> currentBuffer = initiallyEmptyBuffer;

			try {
				for (int i = 0; i < COUNT; i++) {
					if (currentBuffer == null)
						break;                           // stop na null
					Integer item = random.nextInt(100);  // producent generuje nhodn sla

					System.out.println("Produkovno: " + item);
					currentBuffer.add(item);		     // add, remove, isEmpty jsou v java.util.concurrent
					if (currentBuffer.size() == FULL)	 // je-li plny volej exchange
						currentBuffer = exchanger.exchange(currentBuffer);
				}
			} catch (InterruptedException ex) {
				System.out.println("Vada exchange na stran producenta");
			}
			stopLatch.countDown();		                 // dekrementuje ta zvory
		}
	}   

	//to je Konzument
	private static class EmptyingLoop implements Runnable {

		public void run() {
			List<Integer> currentBuffer = initiallyFullBuffer;

			try {
				for (int i = 0; i < COUNT; i++) {
					if (currentBuffer == null)
						break;                           // stop na null
					Integer item = currentBuffer.remove(0);

					System.out.println("Konzumovno " + item);
					sum += item.intValue();              // aby konzument nco delal, st konzumovan poloky
					if (currentBuffer.isEmpty()) {       // volej exchange pi przdnm
						currentBuffer = exchanger.exchange(currentBuffer);
					}
				}
			} catch (InterruptedException ex) {
				System.out.println("Vada exchange u konzumenta");
			}
			stopLatch.countDown();		                // dekrementuje ta zvory
		}
	}   

	public static void main(String args[]) {

		// vytvo buffery
		initiallyEmptyBuffer = new ArrayList<Integer>();	
		initiallyFullBuffer = new ArrayList<Integer>(FULL);

		for (int i = 0; i < FULL; i++) {		        // naplnn bufferu
			initiallyFullBuffer.add(random.nextInt(100));
		}
		new Thread(new FillingLoop()).start();	        // vytvo a spus producenta
		new Thread(new EmptyingLoop()).start();	        // vytvor a spus konzumenta
		try {
			stopLatch.await();		                    // ekn na zvoe
		} catch (InterruptedException ex) {
			ex.printStackTrace();
		}
		System.out.println("Souet vech poloek je " + sum);
	}
}
